

#include "kdialupconfigdlg.h"
#include "wlanlib.h"
#include "kwlansuprocess.h"
#include "globals.h"

#include <qfile.h>
#include <kdebug.h>
#include <qstringlist.h>
#include <qlineedit.h>
#include <qcombobox.h>
//#include <crypt.h>
#include <klocale.h>
#include <qregexp.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <qcheckbox.h>
#include <pwd.h>


KDialupConfigDlg::KDialupConfigDlg(QWidget* parent, const char* name, bool modal, WFlags fl)
: dialupConfig(parent,name, modal,fl)
{
    fillInterfaces(cbConnectionType->currentText());
    m_newProfile = TRUE;
}

KDialupConfigDlg::~KDialupConfigDlg()
{
}

/*$SPECIALIZATION$*/
void KDialupConfigDlg::cancel()
{
    close();
}

void KDialupConfigDlg::ok()
{
    //get username to make him owner of the dialup peer file
    struct passwd *pw;
    uid_t uid;
    QString user;
    uid = geteuid ();
    pw = getpwuid (uid);
    if (pw)
    {
        user = QString(pw->pw_name);
    }
    else {
        KMessageBox::sorry(0,i18n("Could not find out your username. Cannot store the connection."));
        close();
        return;
    }
    QString config = "\"";
    //check input data
    if (cbConnectionType->currentText() == CONNECT_ANALOG) m_data.connectionType = dialupData::ANALOG;
    else if (cbConnectionType->currentText() == CONNECT_ISDN) m_data.connectionType = dialupData::ISDN;
    else m_data.connectionType=dialupData::PPPOE;
    m_data.interface = cbInterface->currentText();
    m_data.user = leUsername->text();
    m_profile = leProfileName->text();
    //TODO: add a Messagbox here
    if (m_profile.isEmpty()) 
    {
        KMessageBox::sorry(0,i18n("You must supply a connection name."));
        return;
    }
    //TODO: Use crypt to encrypt password
    //if (!lePassword->text().isEmpty()) m_data.password=crypt(lePassword->text(),lePassword->text());
    if (!lePassword->text().isEmpty()) m_data.password=lePassword->text();
    m_data.number = lePhoneNumber->text();
    switch (m_data.connectionType)
    {
        case dialupData::ISDN:
            config += "plugin capiplugin.so\n";
            config += "protocoll hdlc\n";
            config += "sync\n";
            config += QString("number %1\n").arg(m_data.number);
            break;
        case dialupData::PPPOE:
            config += QString("plugin rp-pppoe.so %1\n").arg(m_data.interface);
            break;
        case dialupData::ANALOG:
            config += QString("connect \\\"/usr/sbin/chat -v '' ATZ OK-AT-OK \\\\\"ATDT%1\\\\\" CONNECT \\d\\c\\\"\n").arg(m_data.number);
            config += m_data.interface+"\n";
            break;
    }
    m_data.scriptAfterConnect=leScriptAfterConnect->text();
    m_data.scriptBeforeDisconnect = leScriptBeforeDisconnect->text();
    m_data.scriptAfterConnectRoot = cbScriptAfterConnectRoot->isChecked();
    m_data.scriptBeforeDisconnectRoot = cbScriptBeforeDisconnectRoot->isChecked();
    config += QString("user %1\n").arg(m_data.user);
    config += QString("password %1\n").arg(m_data.password);
    QString filename = QString(PPP_PEERS) + QString("/") + m_profile;
    if (m_data.noipdefault)
        config += QString("noipdefault\n");
    if (m_data.defaultroute)
        config += QString("defaultroute\n");
    if (m_data.noauth)
        config += QString("noauth\n");
    if (m_data.persist)
        config += QString("persist\n");
    if (m_data.replacedefaultroute)
        config += QString("replacedefaultroute\n");
    if (m_data.usepeerdns)
        config += QString("usepeerdns\n");
    if (m_data.hidepassword)
        config += QString("hide-password\n");
    config += QString("#scriptafterconnect %1\n").arg(m_data.scriptAfterConnect);
    config += QString("#scriptbeforedisconnect %1\n").arg(m_data.scriptBeforeDisconnect);
    if (m_data.scriptAfterConnectRoot)
        config += QString("#scriptafterconnectroot\n");
    if (m_data.scriptBeforeDisconnectRoot)
        config += QString("#scriptbeforedisconnectroot\n");
    
    config +="\"";
    KWlanSuProcess *proc = new KWlanSuProcess();
    if (!m_newProfile)
        *proc << "rm" << filename << ";";
    *proc << "echo " << config << ">" << filename << ";";
    *proc << "chown" <<  QString("%1:%2").arg(user,user) << filename << ";";
    *proc << "chmod 600" << filename;
    proc->setDescription( i18n("store dialup network"));
    connect(proc, SIGNAL(processExited(SuProcessBase *)),this, SLOT(profileModified(SuProcessBase *)));
    proc->start();
    close();
}

void KDialupConfigDlg::fillInterfaces(const QString& networkType)
{
    cbInterface->clear();
    if (debugOutput) kdDebug() << "Filling interfaces for " << networkType << endl;
    if (networkType==CONNECT_PPPOE){
        QStringList interfaces = getWiredInterfaces();
        for (QStringList::iterator it = interfaces.begin(); it != interfaces.end();it++)
        {
            cbInterface->insertItem(*it);
        }
        cbInterface->setEnabled(TRUE);
        cbInterface->setEditable(FALSE);
        lePhoneNumber->setEnabled(FALSE);
    } else if (networkType==CONNECT_ANALOG)
    {
        cbInterface->insertItem("/dev/ttyS0");
        cbInterface->insertItem("/dev/ttyS1");
        cbInterface->insertItem("/dev/ttyS2");
        cbInterface->insertItem("/dev/ttyS3");
        cbInterface->insertItem("/dev/modem");
        cbInterface->setEnabled(TRUE);
        cbInterface->setEditable(TRUE);
        lePhoneNumber->setEnabled(TRUE);
    }
    // else we have isdn
    else{
        cbInterface->setEnabled(FALSE);
        cbInterface->setEditable(FALSE);
        lePhoneNumber->setEnabled(TRUE);
    } 
}

bool KDialupConfigDlg::init(QString profile)
{
    // mofify existing data;
    bool ok;
    m_profile = profile;
    ok = getDataFromConfig();
    setData();
    leProfileName->setEnabled(FALSE);
    m_newProfile = FALSE;
    return ok;
}

bool KDialupConfigDlg::getDataFromConfig()
{
    if (m_profile.isEmpty()) return FALSE;

    QString filename = QString(PPP_PEERS) + QString("/") + m_profile;
    QFile confFile(filename);
    if ( confFile.open( IO_ReadOnly ) ) {
        QTextStream stream( &confFile );
        QString line;
        
        while ( !stream.atEnd() ) {
            line = stream.readLine();
            parseConfigLine(line);
        }
        confFile.close();
    }
    else {
        KMessageBox::sorry(0,i18n("Could not read connection settings. Maybe you don't have access to it?"));
        return FALSE;
    }
    return TRUE;
}

void KDialupConfigDlg::parseConfigLine( QString line)
{
    // parses a line from the config file and sets the parameters in data
    //skip leading and trailing spaces
    line = line.stripWhiteSpace();
    // check for comment
//    if (line.startsWith("#"))
//        return;
    // now split into parameter / value
    QString param, value;
    int index = line.find (' ');
    if (index==-1)
    {
        param = line;
        value = "";
    }
    else {
        param = line.left(index);
        value = line.mid(index);
        value = value.stripWhiteSpace();
    }
    //now we have the config parameter
    //set connection typ to modem first, as pppoe / isdn contain a plugin parameter
    if (param.lower()=="plugin")
    {
        if (value.lower() == "capiplugin.so") m_data.connectionType = dialupData::ISDN;
        else if (value.lower().startsWith("rp-pppoe.so"))
        {
            m_data.connectionType = dialupData::PPPOE;
            // check the pppoe interface
            index = value.find(' ');
            if (index==-1) kdWarning() << "PPPOE plugin has no interface configured." << endl;
            else m_data.interface = value.mid(index+1);
        }
        return;
    }
    if (param.lower()=="user")
    {
        if (value.startsWith("\""))
            m_data.user = value.mid(1,value.length()-2);
        else m_data.user= value;
        return;
    }
    if (param.lower()=="password")
    {
        if (value.startsWith("\""))
            m_data.password = value.mid(1,value.length()-2);
        else m_data.password= value;
        return;
    }
    if (param.lower()=="number")
        m_data.number = value;
    if (param.lower().startsWith("/dev"))
        m_data.interface = param;
    if (param.lower()=="connect")
    {
        //extract phone number from connection string
        QRegExp regExp;
        regExp.setPattern( "ATDT(\\d*)" );
        if ( regExp.search( value ) > -1 )
            m_data.number = regExp.cap( 1 );

    }
    if (param.lower()=="noipdefault")
    {
        m_data.noipdefault = TRUE;
        return;
    }
    if (param.lower() == "defaultroute")
    {
        m_data.defaultroute = TRUE;
        return;
    }
    if (param.lower()=="replacedefaultroute")
    {
        m_data.replacedefaultroute = TRUE;
        return;
    }
    if (param.lower()=="noauth")
    {
        m_data.noauth = TRUE;
        return;
    }
    if (param.lower()=="persist")
    {
        m_data.persist = TRUE;
        return;
    }
    if (param.lower()=="usepeerdns")
    {
        m_data.usepeerdns = TRUE;
        return;
    }
    if (param.lower()=="hide-password")
    {
        m_data.hidepassword = TRUE;
        return;
    }
    if (param.lower()=="#scriptbeforedisconnect")
    {
        if (value.startsWith("\""))
            m_data.scriptBeforeDisconnect = value.mid(1,value.length()-2);
        else m_data.scriptBeforeDisconnect= value;
        return;
    }
    if (param.lower()=="#scriptafterconnect")
    {
        if (debugOutput) kdDebug() << "Script after connect "<< value << endl;
        if (value.startsWith("\""))
            m_data.scriptAfterConnect = value.mid(1,value.length()-2);
        else m_data.scriptAfterConnect= value;
        return;
    }
    if (param.lower()=="#scriptafterconnectroot")
    {
        m_data.scriptAfterConnectRoot = TRUE;
        return;
    }
    if (param.lower()=="#scriptbeforedisconnectroot")
    {
        m_data.scriptBeforeDisconnectRoot = TRUE;
        return;
    }

    // kdDebug() << "Unknown parameter in peer configuration" << endl;
}

void KDialupConfigDlg::setData()
{
    leProfileName->setText(m_profile);
    leUsername->setText(m_data.user);
    lePhoneNumber->setText(m_data.number);
    leScriptAfterConnect->setText(m_data.scriptAfterConnect);
    leScriptBeforeDisconnect->setText(m_data.scriptBeforeDisconnect);
    cbScriptAfterConnectRoot->setChecked(m_data.scriptAfterConnectRoot);
    cbScriptBeforeDisconnectRoot->setChecked(m_data.scriptBeforeDisconnectRoot);
    switch (m_data.connectionType) {
        case dialupData::ISDN:
            cbConnectionType->setCurrentText(CONNECT_ISDN);
            fillInterfaces( QString(CONNECT_ISDN));
            break;
        case dialupData::PPPOE:
            cbConnectionType->setCurrentText(CONNECT_PPPOE);
            fillInterfaces( QString(CONNECT_PPPOE));
            break;
        case dialupData::ANALOG:
            cbConnectionType->setCurrentText(CONNECT_ANALOG);
            fillInterfaces( QString(CONNECT_ANALOG));
            break;
    }
    if (!m_data.interface.isEmpty())
        cbInterface->setCurrentText(m_data.interface);
}

void KDialupConfigDlg::profileModified(SuProcessBase *proc)
{
    emit(dialupNetworksUpdated());
}
#include "kdialupconfigdlg.moc"

